// Illustrating parameter passing
public class Client3A {
  public static void main(String[] args) {

    String name = "Joe";
    double hourlyRate = 15.50;
    EmployeeV3 manager = new EmployeeV3();                   // (1)
    System.out.println("Manager state before call to setState() method");
    manager.printState();
    manager.setState(name, "Jones", hourlyRate*2.0, false);  // (2)
    System.out.println("Manager state after call to setState() method");
    manager.printState();
    System.out.println();

    System.out.printf("Manager hourly rate before adjusting: %.2f%n",
                       manager.hourlyRate);                  // (3)
    adjustHourlyRate(manager.hourlyRate);     // (4) LOGICAL ERROR!
    System.out.printf("Manager hourly rate after adjusting: %.2f%n",
                       manager.hourlyRate);
    System.out.println();

    EmployeeV3 director = new EmployeeV3();                  // (5)
    System.out.println("Director state before call to copyState() method");
    director.printState();
    copyState(manager, director);                            // (6)
    assert (manager.lastName.equals(director.lastName) &&
            manager.firstName.equals(director.firstName) &&
            manager.hourlyRate == director.hourlyRate &&
            manager.gender == director.gender) :
            "Manager and director have different states after copying.";
    System.out.println("Director state after call to copyState() method:");
    director.printState();
    System.out.println("Manager state after call to copyState() method:");
    manager.printState();
    System.out.println();
  }

  // Method that tries to adjust the hourly rate.
  static void adjustHourlyRate(double hourlyRate) {         // (7)
    hourlyRate = 1.5 * hourlyRate;                          // (8)
    System.out.printf("Adjusted hourlyRate: %.2f%n", hourlyRate);
  }

  // Method that copies the state of one employee over to another employee.
  static void copyState(EmployeeV3 fromEmployee,
                        EmployeeV3 toEmployee) {            // (9)

    toEmployee.setState(fromEmployee.firstName,             // (10)
                        fromEmployee.lastName,
                        fromEmployee.hourlyRate,
                        fromEmployee.gender);

    toEmployee = fromEmployee = null;                       // (11)
  }
}