// A generic utility class for sorting and searching arrays of objects.
class GenericUtility {
  static void sortBySelection(Comparable[] objects) {    // (1)
    // For each pass:
    for (int next = 0; next < objects.length-1; next++) {// (2) outer loop
      // Find the smallest value in the unsorted part of the array
      int smallest = next;
      for (int current = next+1;
           current < objects.length;
           current++) {                                  // (3) inner loop
        if (objects[current].compareTo(objects[smallest]) < 0) { // (4)
          smallest = current;
        }
      }

      assert smallest > next && smallest < objects.length :
        "Index for smallest value " + smallest +
        " should have been between " + (next+1) +
        " and " + objects.length;

      // Swap the smallest value with the first element of the usorted part
      Comparable temp = objects[smallest];                       // (5)
      objects[smallest] = objects[next];
      objects[next] = temp;

      //printArray("Pass #" + (next+1) + ": ", objects);
    }
  }

  static int binarySearch(Comparable[] objects,  int key) {
    int lower = 0;
    int upper = objects.length-1;
    int middle = -1;
    boolean found = false;            // True if the given key is found
    boolean moreElements = true;      // True until whole array is searched
    // Repeat while more elements in array:
    while (moreElements && !found) {        // (1) Main loop
      middle = (lower+upper)/2;       // Middle element in remaining part
      if (lower > upper) {            // Key does not exist in the array
        moreElements = false;
      }
      // If the middle element contains the key:
      //   Return the index of the middle element
      else if (objects[middle].compareTo(key) == 0) { // Key is in
                                                      // the middle element
        found = true;
      }
      // If the middle element is smaller than the key:
      //   Continue searching to the right of the middle element
      else if (objects[middle].compareTo(key) < 0) {// If key exists, it has
         lower = middle + 1;                        // to be in the upper half
      }
      // If the middle element is larger than the key:
      //   Continue searching to the left of the middle element
      else {                             // If key exists, it has to
        upper = middle - 1;              // be in the lower half
      }
    }
    assert (!moreElements || found) : "Error in binary search.";
    if (found)
      return middle;  // The key is in element objects[middle]
    else
      return -1;      // The key is not found
  }

  static int binarySearch(Comparable[] objects,  Comparable key) {
	    int lower = 0;
	    int upper = objects.length-1;
	    int middle = -1;
	    boolean found = false;            // True if the given key is found
	    boolean moreElements = true;      // True until whole array is searched
	    // Repeat while more elements in array:
	    while (moreElements && !found) {        // (1) Main loop
	      middle = (lower+upper)/2;       // Middle element in remaining part
	      if (lower > upper) {            // Key does not exist in the array
	        moreElements = false;
	      }
	      else if (objects[middle].compareTo(key) == 0) { // Key is in
	                                                      // the middle element
	        found = true;
	      }
	      else if (objects[middle].compareTo(key) < 0) {// If key exists, it has
	         lower = middle + 1;                        // to be in the upper half
	      }
	      else {                             // If key exists, it has to
	        upper = middle - 1;              // be in the lower half
	      }
	    }
	    assert (!moreElements || found) : "Error in binary search.";
	    if (found)
	      return middle;  // The key is in element objects[middle]
	    else
	      return -1;      // The key is not found
	  }

  static int linearSearch(Comparable[] objects,  int key) {
    boolean found = false;
    int i = 0;
    // Repeat while more elements in array:
    while (i < objects.length && !found) {  // (1)
      // If the current element contains the key:
      //   Return the index of the current element
      if (objects[i].compareTo(key) == 0) {
        found = true;
      }
      else {
       i++;      // Move to next element
      }
    }
    assert (i >= objects.length || found) : "Error in linear search.";
    if (found)
      return i;  // The key is in element objects[i]
    else
      return -1; // The key is not found
  }

  static void printArray(String prompt, Comparable[] array) {
    System.out.print(prompt);
    for (Comparable element : array) {
      System.out.print(element + " ");
    }
    System.out.println();
  }
}