// Class representing an employee
class Employee {                                                   // (1)

  // Static variable
  final static double NORMAL_WORKWEEK = 37.5;

  // Field variables
  String firstName;
  String lastName;
  double hourlyRate;
  Gender gender;

  // Constructors
  Employee() { }
  Employee(String firstName, String lastName,
           double hourlyRate, Gender gender) {
    this.firstName  = firstName;
    this.lastName   = lastName;
    this.hourlyRate = hourlyRate;
    this.gender     = gender;
  }

  // Determines whether an employee is female.
  boolean isFemale() { return (this.gender == Gender.FEMALE); }

  // Computes the salary of an employee, based on the number of hours
  // worked during the week.
  double computeSalary(double numOfHours) {
    assert numOfHours >= 0 : "Number of hours must be >= 0";
    double weeklySalary = hourlyRate * Employee.NORMAL_WORKWEEK;
    if (numOfHours > Employee.NORMAL_WORKWEEK) {
      weeklySalary += 2.0 * hourlyRate * (numOfHours - NORMAL_WORKWEEK);
    }
    return weeklySalary;
  }

  // Return string representation of the field values of an employee.
  public String toString() {
    return String.format(
      "First name: %-6s Last name: %-8s Hourly rate: %6.2f Gender: %-6s",
      this.firstName, this.lastName, this.hourlyRate, this.gender);
  }
}