/**
 * Class representing a computer player in the game of Nim.
 */
 class ComputerPlayer extends BasicPlayer{
  public String getName(){
    return "intermediate-player";
  }

  public Move nextMove(int rows[]){

    int numFilledRows=0;

    int actualRows[] = new int[rows.length];

    // Finds which rows contain sticks, and creates an actualRows array
    // in the following manner (row 0 and 2 is empty in the rows array):
    // Row1: |||          actualRows[0]: 1
    // Row3: ||    ---->  actualRows[1]: 3
    // Row4: ||||         actualRows[2]: 4
    //
    // Then we know that the first actual row (look at element 0 in rows)
    // is row 1 in rows, second actual row (element 1 in rows) is row 3,
    // and that third actual row is row 4 in rows.
	  for (int i = 0 ; i<rows.length; i++){
        if (rows[i]>0) {
          numFilledRows++;
          actualRows[numFilledRows-1]= i;
        }
      }

    // Creates a new array with only the filled rows (using the actualRows array)
    int newRows[] = new int[numFilledRows];
    for (int i = 0; i< numFilledRows; i++){
      newRows[i]=rows[actualRows[i]];
    }

    // Sorts newRows together with the actualRows array, to easily know which row
    // there are more/fewest sticks in... (esp useful if the methods is to be enhanced
    // to a smarter method)

    int midl, midl2, min;

    // Uses standard selection sort to sort the array
    for(int i=0; i < numFilledRows; i++){
      min = i;

      for(int j=i+1;j< numFilledRows;j++){
        if (newRows[j]< newRows[min])
        min = j;
      }

      midl = newRows[min];
      newRows[min]=newRows[i];
      newRows[i] = midl;

      midl2 = actualRows[min];
      actualRows[min]=actualRows[i];
      actualRows[i] = midl2;

    }

    // If only one row contains sticks
    if (numFilledRows == 1){
      // Removes the whole row (including the last stick).
      return new Move(newRows[0], actualRows[0]);
    }

    // If two rows contains sticks
    if(numFilledRows == 2){
      // If the number of rows are even, we are in a poor situation, and
      // will only remove one stick from one of the rows, hoping for the
      // opponent to make a mistake.
      if (newRows[0] == newRows[1]){
        return new Move(1,actualRows[0]);
      }

      // If not, we remove sticks from the row with the most sticks, so that
      // both rows get the same number of sticks (= loss for the opponent player)
      return new Move(newRows[1]-newRows[0], actualRows[1]);
     }

    // If three rows contains sticks
    if (numFilledRows == 3){
      // Are two rows of equalRows length?
      boolean equalRows = false;
      int diffPos = 0;
      int sameValue=0;
      for(int i=1; i<3; i++){
        // Due to sorting any two equal rows will be next to each other...
        if (newRows[i] == newRows[i-1]) {
          equalRows=true;
          sameValue=newRows[i];
          // diffPos keeps track of which row is different from any other
          diffPos = i+1;
          if (diffPos>2) diffPos=0;
        }
      }

      // If two rows are equal, we remove all sticks from the third row, thereby
      // giving the opponent player two equal rows.
      if(equalRows){
        return new Move(newRows[diffPos], actualRows[diffPos]);
      }

      // Otherwise (all three rows have different number of sticks), we remove
      // one stick from the row with fewest sticks (to prevent two of the rows
      // to get the same number of sticks)
      else{
        // The exception is if one row contains only one stick - then we remove
        // sticks so that the row with most sticks gets less than the row with
        // second-most sticks. This gives three different (length) rows unless
        // the current combination of sticks is 1-2-3, in which case we have
        // lost anyway...
        if (newRows[0]==1)
          return new Move(newRows[2]-newRows[1]+1, actualRows[numFilledRows-1]);
        else
          return new Move(1, actualRows[0]);
      }
    }

    // If there are more than three rows, we simply remove a whole row
    return new Move(newRows[0],actualRows[0]);

  }
}
