/**
 * The class controls a game of Nim.
 */
class Game {
  BasicPlayer player[]; // players of the game
  int rows[];           // stick present on the board
  int currentPlayer;    // player who will make the next move

  /**
   * The constructor accepts the neccesary initialisation parameters for the game.
   *
   * @param player1 - A player object of type BasicPlayer (1st player)
   * @param player2 - A player object of type BasicPlayer (2nd player)
   * @param rows - Overview of how many sticks each row contains
   */
  public Game(BasicPlayer player1, BasicPlayer player2, int[] board){
    player = new BasicPlayer[2];
    player[0]=player1;
    player[1]=player2;
    rows = board;
    currentPlayer = 0;
  }

  /**
   * This method starts the game, and controls the moves.
   *
   * return BasicPlayer - the player who won (may be useful)
   */
  public BasicPlayer start() {
    boolean noWinnerYet = true;
    Move aMove;

    printStatus(rows.length);

    // The game's main loop, executes as long as no winner is found...
    while (noWinnerYet) {
      aMove = player[currentPlayer].nextMove(rows);

      // If the chosen move is legal, we perform it
      if (legal(aMove, rows.length)){
        execute(aMove);
      }
      // If not, the player is disqualified and the other player wins
      else {
	    System.out.println("Sorry, " + player[currentPlayer].getName()
          + ", you are disqualified! The attempted move is not legal.");
        nextPlayer();
        playerWins(currentPlayer);
        return player [currentPlayer];
      }

      // The printStatus method informs if one of the players has won.
      // (Alternatively, we could have made a new method for determining
      // this, since it isn't directly related to printing.)
      noWinnerYet=printStatus(rows.length);

      nextPlayer();
    }

    nextPlayer();
    playerWins(currentPlayer);
    return player[currentPlayer];
  }

  /**
   * This method congratulates the winner.
   *
   * @param winnerNumber - indicates which player won the game
   */
  void playerWins(int winnerNumber) {
    System.out.println("Congratulations " + player[winnerNumber].getName()
                       + ", you won!");
  }

  /**
   * The method changes who is the current player.
   *
   */
  void nextPlayer() {
    currentPlayer++;
    if (currentPlayer==2) currentPlayer=0;
  }

  /**
   * This methods checks is a proposed move is legal.
   *
   * @param aMove - the move to be checked
   * @param antall_rows - indicates how many rows there are in total in the game
   * @return boolean - tells if the given move was legal (true=legal)
   */
  public boolean legal(Move aMove, int numRows) {
    // If the number of sticks (in the given row of the move) is greater than
    // or equal to the number of sticks to be removed in the move, and row number
    // is valid, the move is legal.
    if (aMove.getRowNumber()>=numRows) return false;
    if (rows[aMove.getRowNumber()] >= aMove.getNumSticks() && aMove.getNumSticks()>0)
      return true;
    else
      return false;
  }

  /**
   * This methods accepts a move to be executed, and updates the overview
   * the program keeps on the distribution of sticks on the board.
   * @param aMove - the move to be executed
   */
  public void execute(Move aMove) {
    // Prints information about the move to be executed
    System.out.print(player[currentPlayer].getName() + " removed "
                     + aMove.getNumSticks() + " stick");
    if(aMove.getNumSticks()!=1) System.out.print("s");
    System.out.println(" from row " + aMove.getRowNumber());
    // Update the array of sticks (rows)
    rows[aMove.getRowNumber()] -= aMove.getNumSticks();
  }

  /**
   * This methods prints an overview on the screen of the sticks
   * currently left on the board.
   */
  public boolean printStatus(int numRows) {
    boolean noWinner=false;

    // For each row
    for (int i=0; i<numRows; i++) {
      // If there are sticks in the current row
      if (rows[i]>0) {
        noWinner=true;
        System.out.print("Row " + i +": ");

        // Print a '|' for each stick in the row
        for (int stick=1; stick<=rows[i]; stick++)
          System.out.print("|");

        System.out.println(" (" + rows[i] + ")");
      }
    }

    return noWinner;
  }
}