/**
 * A manager is paid by the hour at a fixed hourly rate.
 * For each hour worked overtime during the week,
 * the hourly rate is doubled.
 * In addition, the manager always receives a fixed manager bonus.
 */
class ManagerV1 extends EmployeeV0 {
  // Field variable
  double managerBonus;

  // Constructors
  /**
   * Creates a manager with default values for all fields.
   */
  ManagerV1() { // explicit default constructor
    super();                                                       // (1)
  }
  /**
   * Creates a manager with the specified name and hourly rate.
   * The manager bonus is set to the default value (0.0).
   * @param firstName First name (and middle names, if any) of the manager.
   * @param lastName Last name of the manager.
   * @param hourlyRate Payment per hour in GBP.
   */
  ManagerV1(String firstName, String lastName, double hourlyRate) {
    super(firstName, lastName, hourlyRate);                        // (2)
  }
  /**
   * Creates a manager with the specified name, hourly rate and manager bonus.
   * @param firstName First name (and middle names, if any) of the manager.
   * @param lastName Last name of the manager.
   * @param hourlyRate Payment per hour in GBP.
   * @param managerBonus A fixed bonus that is added to the weekly salary.
   */
  ManagerV1(String firstName, String lastName,
            double hourlyRate, double bonus) {
    super(firstName, lastName, hourlyRate);                        // (3)
    managerBonus = bonus;                        // Stores the extended state
  }

  // Instance method
  /**
   * Calculates the weekly salary of the manager.
   * This method overrides the method in the EmployeeV0 class.
   * @param numHours Number of hours worked in the current week.
   * @return Weekly salary in GBP for the current week.
   */
  double calculateSalary(double numHours) {                        // (4)
    // Using assert to control values of actual parameters
    assert numHours >= 0 : "Number of hours must be >= 0";
    double fixedSalary = hourlyRate * NORMAL_WORKWEEK;             // (5)
    if (numHours <= NORMAL_WORKWEEK) {
      return fixedSalary + managerBonus;
    } else {
      return fixedSalary
             + 2.0 * hourlyRate * calculateOvertime(numHours)
             + managerBonus;
    }
  }
}