/**
 * This class represents an employee.
 * Subclasses must implement how the weekly salary will be calculated.
 */
abstract class EmployeeVA {
  // Static variable (constant)
  final static double NORMAL_WORKWEEK = 37.5;

  // Field variables
  String firstName;
  String lastName;
  double hourlyRate;

  // Constructors
  /**
   * Creates an employee with given name and hourly rate.
   * @param firstName First name (and middle names, if any) of the employee.
   * @param lastName Last name of the employee.
   * @param hourlyRate Payment per hour| in GBP.
   */
  EmployeeVA(String firstName, String lastName, double hourlyRate) {
    this.firstName = firstName;
    this.lastName = lastName;
    this.hourlyRate = hourlyRate;
  }

  // Abstract method
  /**
   * Calculates the weekly salary for the employee.
   * @param numHours Number of hours worked in the current week.
   * @return Weekly salary in GBP for the current week.
   */
  abstract public double calculateSalary(double numHours);

  // Instance methods
  /**
   * Prints the name and hourly rate for the employee.
   */
  void printState() {
    System.out.printf("Employee %s %s has an hourly rate of %.2f GBP%n",
                      firstName, lastName, hourlyRate);
  }

  // Static method
  /**
   * Calculates number of hours overtime an employee has worked this week.
   * @param numHours Total number of hours worked by an employee.
   * @return Number of overtime hours.
   */
  static double calculateOvertime(double numHours) {
    // Using assert to check the value of the actual parameter
    assert numHours >= 0 : "Number of hours must be >= 0";
    double overtime = 0.0; // assume no overtime
    if (numHours > NORMAL_WORKWEEK) {
      overtime = numHours - NORMAL_WORKWEEK;
    }
    assert overtime >= 0 : "Number of overtime hours must be >= 0";
    return overtime;
  }
}