/**
 * An hourly worker is paid for each hour she/he has worked in
 * the current week, based on a fixed rate per hour.
 */
public class HourlyWorker extends Employee {
  // Constructors
  /**
   * Creates an hourly worker with the given name and hourly rate.
   * @param firstName First name (and any middle names) of the worker.
   * @param lastName Last name of the worker.
   * @param hourlyRate Payment per hour in GBP.
   */
  public HourlyWorker(String firstName, String lastName, double hourlyRate) {
    super(firstName, lastName, hourlyRate);
  }

  // Instance methods
  /**
   * Calculates the weekly salary for the hourly worker.
   * This method overrides the method from the Employee class.
   * @param numHours The number of hours worked in the current week.
   * @return Weekly salary in GBP for the current week.
   *
   * @pre numHours >= 0.0
   * @post return >= 0.0
   */
  public double calculateSalary(double numHours) {
    assert numHours >= 0.0 : "Number of hours must be >= 0.0";       // @pre
    double salary = numHours * hourlyRate;
    assert salary >= 0.0 : "Weekly salary must be >= 0.0";           // @post
    return salary;
  }
  /**
   * Compares two objects for equality, i.e the current object and
   * the object whose reference value is passed in the parameter.
   * This method overrides the method in the Employee class.
   * @param other The object the current object will be compared with.
   * @return true If the two objects have the same state; otherwise false.
   */
  public boolean equals(Object other) {
    if (other == this) return true;                    // Same object?
    if (!(other instanceof HourlyWorker)) return false;// dynamic type control
    HourlyWorker emp = (HourlyWorker) other;           // downcasting
    return firstName.equals(emp.firstName)         // using equals() in String
           && lastName.equals(emp.lastName)
           && hourlyRate == emp.hourlyRate;
  }

  /**
   * Creates a string representation of an hourly worker.
   * @return A string representing the current state of an hourly worker.
   */
  public String toString() { // [HOW IS THIS METHOD DIFFERENT FROM Employee?]
    return String.format(
                  "First name: %s\tLast name: %s\tHourly rate: %.2f GBP",
                  firstName, lastName, hourlyRate);
  }
}
