/**
 * An hourly worker is paid for each hour they have worked in
 * the current week, based on a fixed rate per hour.
 */
class HourlyWorkerV0 extends EmployeeV0 {
  // Constructors
  /**
   * Creates an hourly worker with default values from the superclass.
   */
  public HourlyWorkerV0() {  // explicit default constructor
    super();
  }
  /**
   * Creates an hourly worker with the given name and hourly rate.
   * @param firstName First name (and any middle names) of the worker.
   * @param lastName Last name of the worker.
   * @param hourlyRate Payment per hour in GBP.
   */
  public HourlyWorkerV0(String firstName, String lastName,
                        double hourlyRate) {
    super(firstName, lastName, hourlyRate);
  }

  // Instance method
  /**
   * Calculates the weekly salary for the hourly worker.
   * This method overrides the method in the EmployeeV0 class.
   * @param numHours The number of hours worked in the current week.
   * @return Weekly salary in GBP for the current week.
   */
  public double calculateSalary(double numHours) {
    // Using assert to check the value of the actual parameter.
    assert numHours >= 0 : "Number of hours must be >= 0";
    return numHours * hourlyRate;
  }
}