import java.util.Random;
/**
 * Represents a row in a lotto draw, consisting of MAX_COUNT numbers.
 */
class LottoRow {
  // Constants
  static final int SMALLEST_NUMBER = 1;
  static final int LARGEST_NUMBER = 34;
  static final int MAX_COUNT = 7;

  // Field variables
  boolean[] drawn;    // true means number equal to index+1 has been drawn
  int[] lottoNumbers; // array (row) holding the numbers drawn
  int numDrawn;       // how many numbers have been drawn so far

  // Constructors
  /**
   * Creates and initialises a lotto row. No numbers are drawn yet,
   * so the row does not contain any numbers.
   */
  public LottoRow() { // Problem 1: Create and initialise a lotto row.
    drawn = new boolean[LARGEST_NUMBER];  // false
    lottoNumbers = new int[MAX_COUNT];    // 0
    numDrawn = 0;
  }

  /**
   * Creates and initialises a lotto row, copying numbers from an array
   * of integers, passed as parameter.
   * @param row This lotto row will be copied to the LottoRow object.
   */
  public LottoRow(int[] row) { // Problem 2: Create and initialise a lotto row
                               // using numbers passed in an array
    drawn = new boolean[LARGEST_NUMBER];  // false
    lottoNumbers = new int[MAX_COUNT];    // 0
    for (int i=0; i<MAX_COUNT; i++) {
      lottoNumbers[i] = row[i];
      drawn[row[i]-1] = true;
    }
    numDrawn = row.length;
  }

  // Instance methods
  /**
   * Makes a new lotto draw.
   */
  public void makeALottoDraw() { // Problem 3: Draw a randow lotto row.
    Random lotto = new Random();
    numDrawn = 0;          // In case the draw is repeated
    int nextNumber;
    while (numDrawn < MAX_COUNT) {
       nextNumber = lotto.nextInt(LARGEST_NUMBER) + 1;
       if (!drawn[nextNumber-1]) {
         drawn[nextNumber-1] = true;
         lottoNumbers[numDrawn] = nextNumber;
         numDrawn++;
       }
    }
  }

  /**
   * Gets the i-th 'ball' of the current lotto row, i.e. the i-th number.
   * @param i Index of the desired number, from 0 to MAX_COUNT-1.
   * @return The number of the i-th ball, or 0 if given index is invalid.
   */
  public int getBall(int i) {// Problem 4: Get the i-th number in the lotto row.
    // Using assert to check the value of the actual parameter
    assert ((i >= 0) && (i < MAX_COUNT))
           : "Index of lotto ball must be 0 <= i < " + MAX_COUNT;
    return lottoNumbers[i];
  }

  /**
   * Compares two objects for equality, i.e the current object and
   * the object whose reference value is passed in the parameter.
   * This method overrides the method in the Object class.
   * @param other The object the current object will be compared with.
   * @return true If the two rows have the same state; otherwise false.
   */
  public boolean equals(Object other) {  // Problem 5: Two lotto rows equal?
    if (other == this) return true;                    // Same object?
    if (!(other instanceof LottoRow)) return false;    // dynamic type control
    LottoRow row = (LottoRow) other; // downcasting
    for (int i = 0; i < MAX_COUNT; i++) {
      if (lottoNumbers[i] != row.lottoNumbers[i])
        return false;
    }
    return true;
  }

  /**
   * Creates a string representation of the object.
   * This method overrides the method in the Object class.
   * @return A string representing the current state of the object.
   */
  public String toString() { //Problem 6: Create a string representing the lotto row.
    String numberString = "";
    for (int i = 0; i < numDrawn; i++) {
      if (i > 0) {
        numberString += ", ";
      }
      numberString += lottoNumbers[i];
    }
    return String.format("Lotto row: %s (%d numbers drawn)",
                         numberString, numDrawn);
  }
}