/**
 * A manager is paid per hour worked in the current week, plus a
 * fixed manager bonus.
 */
class Manager extends Employee {
  // Field variable
  double managerBonus;

  // Constructors
  /**
  * Creates a manager with given name, hourly rate and manager bonus.
   * @param firstName First name (and middle names, if any) of the manager.
   * @param lastName Last name of the manager.
   * @param hourlyRate Payment per hour in GBP.
   * @param managerBonus A bonus added to the weekly salary.
   */
  public Manager(String firstName, String lastName, double hourlyRate,
                      double bonus) {
    super(firstName, lastName, hourlyRate);                          // (1)
    managerBonus = bonus;
  }

  // Instance methods
  /**
   * Calculates the weekly salary of the manager.
   * @param numHours Number of hours worked in current week.
   * @return Weekly salary in GBP for the current week.
   *
   * @pre numHours >= 0.0
   * @post return >= 0.0
   */
  double calculateSalary(double numHours) {                          // (2)
    assert numHours >= 0.0 : "Number of hours must be >= 0.0";       // @pre
    double salary = super.calculateSalary(numHours);                 // (3)
    salary += managerBonus;                                          // (4)
    assert salary >= 0.0 : "Weekly salary must be >= 0.0";           // @post
    return salary;
  }

  // Overridden methods from Object
  /**
   * Compares the object with another object of the same type.
   * @param obj The object the current object will be compared with.
   * @return true if the objected referred to by obj is of the same type as
   * the current object and the two objects have the same state; false otherwise.
   */
  public boolean equals(Object obj) {                                // (5)
    if (obj instanceof Manager) {
      Employee emp = (Employee) obj;
      Manager leader = (Manager) obj;
      return super.equals(emp)&& managerBonus == leader.managerBonus;
    }
    else
      return false;
  }

  /**
   * Creates a string representation of the object.
   * @return A string representing the current state of the object.
   */
  public String toString() {                                         // (6)
    String report = String.format("%s%nManager bonus: %.2f GBP",
                                   super.toString(), managerBonus);
    return report;
  }
}