/**
 * A manager is paid by the hour at a fixed hourly rate.
 * For each hour worked overtime during the week,
 * the hourly rate is doubled.
 * In addition, the manager always receives a fixed manager bonus.
 */
class ManagerV2 extends EmployeeV0 {
  // Field variable
  double managerBonus;

  // Constructors
  /**
   * Creates a manager with default values for all fields.
   */
  ManagerV2() { // explicit default constructor
    super();                                                       // (1)
  }
  /**
   * Creates a manager with given name and hourly rate.
   * The manager bonus is set to the default value (0.0).
   * @param firstName First name (and middle names, if any) of the manager.
   * @param lastName Last name of the manager.
   * @param hourlyRate Payment per hour in GBP.
   */
  ManagerV2(String firstName, String lastName, double hourlyRate) {
    super(firstName, lastName, hourlyRate);                        // (2)
  }
  /**
   * Creates a manager with given name, hourly rate and manager bonus.
   * @param firstName First name (and middle names, if any) of the manager.
   * @param lastName Last name of the manager.
   * @param hourlyRate Payment per hour in GBP.
   * @param managerBonus A fixed bonus that is added to the weekly salary.
   */
  ManagerV2(String firstName, String lastName,
            double hourlyRate, double bonus) {
    super(firstName, lastName, hourlyRate);                        // (3)
    managerBonus = bonus;                        // Stores the extended state
  }

  // Instance methods
  /**
   * Calculates the weekly salary of the manager.
   * This method overrides the method in the EmployeeV0 class.
   * @param numHours Number of hours worked in the current week.
   * @return Weekly salary in GBP for the current week.
   */
  double calculateSalary(double numHours) {                        // (4)
    // Using assert to control values of actual parameters
    assert numHours >= 0 : "Number of hours must be >= 0";
    double fixedSalary = super.calculateSalary(numHours);          // (5)
    fixedSalary += managerBonus;                                   // (6)
    return fixedSalary;
  }
  /**
   * Prints name, hourly salary and bonus for the manager.
   * This method overrides the method in the EmployeeV0 class.
   */
  void printState() {                                              // (7)
    System.out.printf("Manager %s %s has an hourly rate of %.2f GBP%n",
             super.firstName, super.lastName, super.hourlyRate);   // (8)
    System.out.printf("and a bonus of %.2f GBP for %.1f hours work%n",
                      managerBonus, super.NORMAL_WORKWEEK);        // (9)
  }
}