/**
 * A piece worker is paid for each unit she/he produces.
 */
class PieceWorker extends Employee {
  // Field variables
  double payPerUnit;
  int numUnits;

  // Constructors
  /**
   * Creates a piece worker with given name, payment per unit and
   * number of units produced in the current week.
   */
  public PieceWorker(String firstName, String lastName,
                     double pay, int count) {
    super(firstName, lastName, 0.0);
    payPerUnit = pay;              // stores extended state
    numUnits = count;
  }

  // Instance methods
  /**
   * Calculates the weekly salary for the piece worker.
   * @param numHours The number of hours worked in the current week.
   * This parameter is not used, but must be given for the class to
   * satisfy the same contract as the superclass.
   * @return Weekly salary in GBP for the current week.
   *
   * @pre numHours >= 0.0
   * @post return >= 0.0
   */
  public double calculateSalary(double numHours) {
	assert numHours >= 0.0 : "Number of hours must be >= 0.0";       // @pre
	double salary = calculateSalary();
	assert salary >= 0.0 : "Weekly salary must be >= 0.0";           // @post
    return salary;
  }
  /**
   * Calculates the weekly salary for the piece worker. This method
   * extends the interface of the superclass.
   * @return Weekly salary in GBP for the current week.
   *
   * @pre numUnits >= 0 && payPerUnit >= 0.0
   * @post return >= 0.0
   */
  public double calculateSalary() {
    assert numUnits >= 0 : "Number of units produced must be >= 0";  // @pre
    assert payPerUnit >= 0.0 : "Payment per unit must be >= 0.0";    // @pre
    double salary = numUnits * payPerUnit;
    assert salary >= 0.0 : "Weekly salary must be >= 0.0";           // @post
    return salary;
  }

  // Overridden methods from Object
  /**
   * Compares the object with another object of the same type.
   * @param obj The object the current object will be compared with.
   * @return true if the objected referred to by obj is of the same type as
   * the current object and the two objects have the same state; false otherwise.
   */
  public boolean equals(Object obj) {
    if (obj instanceof PieceWorker) {
      Employee emp = (Employee) obj;
      PieceWorker pieceWorker = (PieceWorker) obj;
      return super.equals(emp)
             && payPerUnit == pieceWorker.payPerUnit
             && numUnits == pieceWorker.numUnits;
    }
    else
      return false;
  }

  /**
   * Creates a string representation of the object.
   * @return A string representing the current state of the object.
   */
  public String toString() {
    String report = super.toString() +
                    "\nNumber of units: " + numUnits +
                    "\nPayment per unit: " + payPerUnit;
    return report;
  }
}