/**
 * A piece worker is paid for each unit they produce.
 */
class PieceWorkerV0 extends EmployeeV0 {
  // Field variables
  double payPerUnit;
  int numUnits;

  // Constructors
  /**
   * Creates a piece worker with default values.
   */
  public PieceWorkerV0() {  // explicit default constructor
    super();
  }
  /**
   * Creates a piece worker with the given name.
   * All other fields are set to default values implicitly.
   */
  public PieceWorkerV0(String firstName, String lastName) {
    super(firstName, lastName, 0.0);
  }
  /**
   * Creates a piece worker with the given name, payment per unit and
   * number of units produced in the current week.
   */
  public PieceWorkerV0(String firstName, String lastName,
                       double pay, int count) {
    super(firstName, lastName, 0.0);
    payPerUnit = pay;              // Stores extended state.
    numUnits = count;
  }

  // Instance methods
  /**
   * Calculates the weekly salary for the piece worker.
   * This method overrides the method in the EmployeeV0 class.
   * @param numHours The number of hours worked in the current week.
   * This parameter is not used, but must be given for the class to
   * satisfy the same contract as the superclass.
   * @return Weekly salary in GBP for the current week.
   */
  public double calculateSalary(double numHours) {
    // Using assert to check the value of the actual parameter
    assert numHours >= 0 : "Number of hours must be >= 0";
    return calculateSalary();
  }
  /**
   * Prints the name, payment per unit and number of units produced
   * for the piece worker.
   * This method overrides the method in the EmployeeV0 class.
   */
  void printState() {
    super.printState();
    System.out.printf("Has produced %d unit at %.2f GBP/piece%n",
                      numUnits, payPerUnit);
  }
  /**
   * Calculates the weekly salary for the piece worker.
   * This method extends the contract of the superclass.
   * @return Weekly salary in GBP for the current week.
   */
  public double calculateSalary() {
    return numUnits * payPerUnit;
  }
}