import java.util.Scanner;
/**
 * Allows user to input information from the keyboard.
 */
class TextUserInterface {
  Scanner input = new Scanner(System.in);

  /**
   * Method that allows the user to input a string.
   * @param wantedInfo Prompt for the expected string.
   * @return The string typed at the keyboard.
   */
  String requestString(String wantedInfo) {
    System.out.printf("- %s: ", wantedInfo);
    return input.nextLine();
  }

  /**
   * Method that allows the user to input a floating-point value.
   * Note: Only one number per line is allowed.
   * @param wantedInfo Prompt for the expected decimal number.
   * @return The decimal value of the number typed.
   */
  double requestDouble(String wantedInfo) {
    System.out.printf("- %s: ", wantedInfo);
    double value = input.nextDouble();
    input.nextLine(); // skip the rest of the line
    return value;
  }

  /**
   * Method that allows the user to input an integer value.
   * Note: Only one number per line is allowed.
   * @param wantedInfo Prompt for the expected integer number.
   * @return The integer value of the number typed.
   */
  int requestInteger(String wantedInfo) {
    System.out.printf("- %s: ", wantedInfo);
    int value = input.nextInt();
    input.nextLine(); // skip the rest of the line
    return value;
  }

  /**
   * Method that allows the user to input a y(es) or n(o) reply.
   * @param wantedInfo Prompt for the expected answer.
   * @return true if answer "y" is given, false otherwise.
   */
  boolean requestYesNoAnswer(String wantedInfo) {
    String answer = requestString(wantedInfo + " (y/n)");
    return answer.equals("y");
  }

  /**
   * Method that allows the user to type employee data.
   * Note: Employees must be of type Employee, HourlyWorker, Manager
   * or PieceWorker.
   * The user selects which type of employee to create.
   * @param label Label identifying the employee.
   * @return An employee object of the chosen type.
   */
  Employee inputEmployee(String label) {
    System.out.printf("Type information about %s:\n", label);
    // Select type of object
    int empCode = 0;
    do {
      System.out.println("Supported employee types are:");
      System.out.print(" 1 - Employee. 2 - Hourly worker.");
      System.out.println(" 3 - Manager. 4 - Piece worker");
      String empType = requestString("Employee type");
      empCode = Integer.parseInt(empType);
    } while (empCode < 1 && empCode > 4);

    // Read data common to all types
    String firstName = requestString("First name");
    String lastName = requestString("Last name");

    // Read data specific to some of the employee types
    double hourlyRate = 0.0, bonus = 0.0, payPerUnit = 0.0;
    int numUnits = 0;
    if (empCode == 4) { // Piece Worker
        payPerUnit = requestDouble("Payment per unit");
      numUnits = requestInteger("Number of units produced");
    } else { // Employee, Manager or HourlyWorker
      hourlyRate = requestDouble("Hourly rate");
    }
    if (empCode == 3) { // Manager
      bonus = requestDouble("Manager bonus");
    }

    // Create and return an object of the correct type
    Employee newEmp = null;
    switch (empCode) {
    case 1:
      newEmp = new Employee(firstName, lastName, hourlyRate);
      break;
    case 2:
      newEmp = new HourlyWorker(firstName, lastName, hourlyRate);
      break;
    case 3:
      newEmp = new Manager(firstName, lastName, hourlyRate, bonus);
      break;
    case 4:
      newEmp = new PieceWorker(firstName, lastName, payPerUnit, numUnits);
      break;
    }
    return newEmp;
  }
}