// File: TestGameBoard.java
public class TestGameBoard {
  public static void main(String[] args) {
    testEmptyCell();
    testDropPiece();
    testStacking();
  }

  public static void testEmptyCell() {
    GameBoard board = new GameBoard();
    // Bottom row has index 0.
    int row = 0;
    // The leftmost column also has index 0
    int column = 0;
    assert board.pieceAt(column, row) == GameBoard.EMPTY_CELL;
  }

  public static void testDropPiece() {
    GameBoard board = new GameBoard();
    board.dropPieceDownColumn('X', 0);
    board.dropPieceDownColumn('O', 2);
    /* The bottom rows should now look like this:

       1 | . . . . . . . |
       0 | X . O . . . . |
         -----------------
        /  0 1 2 3 4 5 6  \
    */
    assert board.pieceAt(0, 0) == 'X';
    assert board.pieceAt(1, 0) == GameBoard.EMPTY_CELL;
    assert board.pieceAt(2, 0) == 'O';
    assert board.pieceAt(0, 1) == GameBoard.EMPTY_CELL;
  }

  static void assertEquals(String message, char expected, char actual) {
    if (expected != actual) {
      System.err.printf("Not equal: %s: Expected <%s>, but got <%s>\n",
          message, expected, actual);
    }
    assert expected == actual;
  }

  public static void testStacking() {
    GameBoard board = new GameBoard();
    board.dropPieceDownColumn('X', 3);
    board.dropPieceDownColumn('O', 3);

    /* The bottom rows should now look like this:

       2 | . . . . . . . |
       1 | . . . O . . . |
       0 | . . . X . . . |
         -----------------
        /  0 1 2 3 4 5 6  \
    */
    assertEquals("First dropped", 'X', board.pieceAt(3, 0));
    assertEquals("Second dropped", 'O', board.pieceAt(3, 1));
    assertEquals("Only two stacked", GameBoard.EMPTY_CELL, board.pieceAt(3, 2));
  }
}