// File: TestGameBoard.java
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.assertFalse;
import org.junit.Test;

public class TestGameBoard {
  @Test
  public void testEmptyCell() {
    GameBoard board = new GameBoard();
    // Bottom row has index 0.
    int row = 0;
    // The leftmost column also has index 0
    int column = 0;
    assertEquals("Empty", GameBoard.EMPTY_CELL, board.pieceAt(column, row));
  }

  @Test
  public void testDropPiece() {
    GameBoard board = new GameBoard();
    board.dropPieceDownColumn('X', 0);
    board.dropPieceDownColumn('O', 2);
    /* The bottom rows should now look like this:

       1 | . . . . . . . |
       0 | X . O . . . . |
         -----------------
        /  0 1 2 3 4 5 6  \
    */
    assertEquals("Cell 0,0", 'X', board.pieceAt(0, 0));
    assertEquals("Cell 1,0", GameBoard.EMPTY_CELL, board.pieceAt(1, 0));
    assertEquals("Cell 2,0", 'O', board.pieceAt(2, 0));
    assertEquals("Cell 0,1", GameBoard.EMPTY_CELL, board.pieceAt(0, 1));
  }

  @Test
  public void testStacking() {
    GameBoard board = new GameBoard();
    board.dropPieceDownColumn('X', 3);
    board.dropPieceDownColumn('O', 3);

    /* The bottom rows should now look like this:

       2 | . . . . . . . |
       1 | . . . O . . . |
       0 | . . . X . . . |
         -----------------
        /  0 1 2 3 4 5 6  \
    */
    assertEquals("First dropped", 'X', board.pieceAt(3, 0));
    assertEquals("Second dropped", 'O', board.pieceAt(3, 1));
    assertEquals("Only two stacked", GameBoard.EMPTY_CELL, board.pieceAt(3, 2));
  }
}