// File: GameBoard.java
public class GameBoard {
  public static final int NO_VACANT_ROW = -1;
  public static final char EMPTY_CELL = '\u0000';

  char[][] cellContents = new char[7][6];

  GameBoard() {
    assert cellContents[0][0] == EMPTY_CELL;
  }

  GameBoard(String move) {
    for (int i = 0; i < move.length(); ++i) {
      boolean isEvenNumberedMove = i % 2 == 0;
      char piece = 'X';
      if (!isEvenNumberedMove)
        piece = 'O';
      String digit = move.substring(i, i + 1);
      int column = Integer.parseInt(digit);
      dropPieceDownColumn(piece, column);
    }
  }

  char pieceAt(int column, int row) {
    return cellContents[column][row];
  }

  int findBottommostVacantRow(int column) {
    char[] columnContents = cellContents[column];
    for (int row = 0; row < columnContents.length; ++row) {
      if (columnContents[row] == EMPTY_CELL)
        return row;
    }
    return NO_VACANT_ROW;
  }

  void dropPieceDownColumn(char piece, int column) {
    cellContents[column][findBottommostVacantRow(column)] = piece;
  }

  boolean isColumnFull(int column) {
    char[] columnContents = cellContents[column];
    int indexOfTopmostCell = columnContents.length - 1;
    char topmostCell = columnContents[indexOfTopmostCell];
    return topmostCell != EMPTY_CELL;
  }
}