// File: TestGameBoard.java
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.assertFalse;
import org.junit.Test;

public class TestGameBoard {
  @Test
  public void testEmptyCell() {
    GameBoard board = new GameBoard();
    // Bottom row has index 0.
    int row = 0;
    // The leftmost column also has index 0
    int column = 0;
    assertEquals("Empty", GameBoard.EMPTY_CELL, board.pieceAt(column, row));
  }

  @Test
  public void testDropPiece() {
    GameBoard board = GameSimulator.buildGameBoard("02");
    /* The bottom rows should now look like this:

       1 | . . . . . . . |
       0 | X . O . . . . |
         -----------------
        /  0 1 2 3 4 5 6  \
    */
    assertEquals("Cell 0,0", 'X', board.pieceAt(0, 0));
    assertEquals("Cell 1,0", GameBoard.EMPTY_CELL, board.pieceAt(1, 0));
    assertEquals("Cell 2,0", 'O', board.pieceAt(2, 0));
    assertEquals("Cell 0,1", GameBoard.EMPTY_CELL, board.pieceAt(0, 1));
  }

  @Test
  public void testStacking() {
    GameBoard board = GameSimulator.buildGameBoard("33");
    /* The bottom rows should now look like this:

       2 | . . . . . . . |
       1 | . . . O . . . |
       0 | . . . X . . . |
         -----------------
        /  0 1 2 3 4 5 6  \
    */
    assertEquals("First dropped", 'X', board.pieceAt(3, 0));
    assertEquals("Second dropped", 'O', board.pieceAt(3, 1));
    assertEquals("Only two stacked", GameBoard.EMPTY_CELL, board.pieceAt(3, 2));
  }

  @Test
  public void testFullColumn() {
    GameBoard board = new GameBoard();
    int column = 3;
    char[] pieces = {'X', 'O', 'X', 'O', 'X', 'O'};
    for (char piece : pieces) {
      assertFalse(board.isColumnFull(column));
      board.dropPieceDownColumn(piece, column);
    }
    assertTrue(board.isColumnFull(column));
  }

  @Test
  public void testIsValidColumnSelection() {
    GameBoard board = GameSimulator.buildGameBoard("111111");
    assertFalse(board.isValidColumnSelection(-1));
    assertTrue(board.isValidColumnSelection(0));
    assertFalse(board.isValidColumnSelection(1));
    assertTrue(board.isValidColumnSelection(6));
    assertFalse(board.isValidColumnSelection(7));
  }

  @Test
  public void testSimpleGameStatus() {
    GameBoard start = new GameBoard();
    assertFalse(start.isGameOver());
    GameBoard inProgress = GameSimulator.buildGameBoard("340222244");
    assertFalse(inProgress.isGameOver());
    GameBoard lastMove = GameSimulator.buildGameBoard(
          "00000011111122222243333334444455555566666");
    assertFalse(lastMove.isGameOver());
    GameBoard noWinner = GameSimulator.buildGameBoard(
          "000000111111222222433333344444555555666666");
    assertTrue(noWinner.isGameOver());
  }
}