// File: GameBoard.java
public class GameBoard {
  public static final int NO_VACANT_ROW = -1;
  public static final char EMPTY_CELL = '\u0000';

  char[][] cellContents = new char[7][6];

  GameBoard() {
    assert cellContents[0][0] == EMPTY_CELL;
  }

  char pieceAt(int column, int row) {
    return cellContents[column][row];
  }

  int findBottommostVacantRow(int column) {
    char[] columnContents = cellContents[column];
    for (int row = 0; row < columnContents.length; ++row) {
      if (columnContents[row] == EMPTY_CELL)
        return row;
    }
    return NO_VACANT_ROW;
  }

  int moveCounter;

  void dropPieceDownColumn(char piece, int column) {
    cellContents[column][findBottommostVacantRow(column)] = piece;
    ++moveCounter;
  }

  boolean isColumnFull(int column) {
    char[] columnContents = cellContents[column];
    int indexOfTopmostCell = columnContents.length - 1;
    char topmostCell = columnContents[indexOfTopmostCell];
    return topmostCell != EMPTY_CELL;
  }

  boolean isValidColumnSelection(int column) {
    if ((column < 0) || (column > 6))
      return false;
    return !isColumnFull(column);
  }

  char nextPiece() {
    boolean isEvenNumberedMove = moveCounter % 2 == 0;
    if (isEvenNumberedMove) {
      return 'X';
    } else {
      return 'O';
    }
  }

  boolean isGameOver() {
    return moveCounter >= 42;
  }
}