// File: GameTextUserInterface.java
import java.util.Scanner;

public class GameTextUserInterface {
  GameBoard board;

  static String gameBoardRowAsText(GameBoard board, int row) {
    final int NUMBER_OF_COLUMNS = 7;
    final int LAST_COLUMN = NUMBER_OF_COLUMNS - 1;
    String rowText = "|";
    for (int column = 0; column < LAST_COLUMN; ++column)
      rowText += gameBoardCellAsText(board, column, row) + " ";
    return rowText + gameBoardCellAsText(board, LAST_COLUMN, row) + "|";
  }

  static char gameBoardCellAsText(GameBoard board, int column, int row) {
    char state = board.pieceAt(column, row);
    if (state == GameBoard.EMPTY_CELL)
      return '.';
    return state;
  }

  static String gameBoardAsText(GameBoard board) {
    String text = " 0 1 2 3 4 5 6\n";
    final int TOPMOST_ROW = 5;
    for (int row = TOPMOST_ROW; row >= 0; --row)
      text += gameBoardRowAsText(board, row) + "\n";
    return text + "===============\n";
  }

  public void showGameBoard() {
    System.out.print(gameBoardAsText(board));
  }

  public static void main(String[] args) {
    GameTextUserInterface ui = new GameTextUserInterface();
    ui.runGameLoop();
  }

  GameTextUserInterface() {
    board = new GameBoard();
  }

  void runGameLoop() {
    showGameBoard();
    while (!board.isGameOver()) {
      playOneMove();
      showGameBoard();
    }
  }

  void playOneMove() {
    char piece = board.nextPiece();
    showNextPiece(piece);
    int column = requestColumnSelection();
    board.dropPieceDownColumn(piece, column);
  }

  void showNextPiece(char piece) {
    System.out.printf("Next piece: %s\n", piece);
  }

  int requestColumnSelection() {
    Scanner scanner = new Scanner(System.in);
    while (true) {
      System.out.print("Select a column [0-6]: ");

      if (scanner.hasNextInt()) {
        int column = scanner.nextInt();
        if (board.isValidColumnSelection(column)) {
          scanner.nextLine(); // Skip the rest of the line.
          return column;
        }
      }
      scanner.nextLine(); // Skip the rest of the line.
      System.out.println("Invalid column. Try again.");
    }
  }
}