package game.test;    // File: game/test/TestGameBoard.java

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.assertFalse;
import org.junit.Test;

import game.GameBoard;

public class TestGameBoard {
  @Test
  public void testEmptyCell() {
    GameBoard board = GameSimulator.buildGameBoard("");
    // Bottom row has index 0.
    int row = 0;
    // The leftmost column also has index 0
    int column = 0;
    assertEquals("Empty", GameSimulator.EMPTY_CELL, board.pieceAt(column, row));
  }

  @Test
  public void testDropPiece() {
    GameBoard board = GameSimulator.buildGameBoard("02");
    /* The bottom rows should now look like this:

       1 | . . . . . . . |
       0 | X . O . . . . |
         -----------------
        /  0 1 2 3 4 5 6  \
    */
    assertEquals("Cell 0,0", GameSimulator.X_PIECE, board.pieceAt(0, 0));
    assertEquals("Cell 1,0", GameSimulator.EMPTY_CELL, board.pieceAt(1, 0));
    assertEquals("Cell 2,0", GameSimulator.O_PIECE, board.pieceAt(2, 0));
    assertEquals("Cell 0,1", GameSimulator.EMPTY_CELL, board.pieceAt(0, 1));
  }

  @Test
  public void testStacking() {
    GameBoard board = GameSimulator.buildGameBoard("33");
    /* The bottom rows should now look like this:

       2 | . . . . . . . |
       1 | . . . O . . . |
       0 | . . . X . . . |
         -----------------
        /  0 1 2 3 4 5 6  \
    */
    assertEquals("First dropped", GameSimulator.X_PIECE, board.pieceAt(3, 0));
    assertEquals("Second dropped", GameSimulator.O_PIECE, board.pieceAt(3, 1));
    assertEquals("Only two stacked", GameSimulator.EMPTY_CELL,
                 board.pieceAt(3, 2));
  }

  @Test
  public void testFullColumn() {
    GameBoard board = GameSimulator.buildGameBoard("");
    int column = 3;
    for (int i = 0; i < GameBoard.ROW_COUNT; ++i) {
      assertFalse(board.isColumnFull(column));
      board.dropPieceDownColumn(board.nextPiece(), column);
    }
    assertTrue(board.isColumnFull(column));
  }

  @Test
  public void testIsValidColumnSelection() {
    GameBoard board = GameSimulator.buildGameBoard("111111");
    assertFalse(board.isValidColumnSelection(-1));
    assertTrue(board.isValidColumnSelection(0));
    assertFalse(board.isValidColumnSelection(1));
    assertTrue(board.isValidColumnSelection(6));
    assertFalse(board.isValidColumnSelection(7));
  }

  @Test
  public void testSimpleGameStatus() {
    GameBoard start = GameSimulator.buildGameBoard("");
    assertFalse(start.isGameOver());
    GameBoard inProgress = GameSimulator.buildGameBoard("340222244");
    assertFalse(inProgress.isGameOver());
    GameBoard lastMove = GameSimulator.buildGameBoard(
        "00000011111122222243333334444455555566666");
    assertFalse(lastMove.isGameOver());
    GameBoard noWinner = GameSimulator.buildGameBoard(
        "000000111111222222433333344444555555666666");
    assertTrue(noWinner.isGameOver());
  }

  @Test
  public void testWinnerStatus() {
    GameBoard xWinnerVertical = GameSimulator.buildGameBoard( "2324252");
    assertTrue(xWinnerVertical.isGameOver());
    GameBoard oWinnerHorizontal = GameSimulator.buildGameBoard( "06142335");
    assertTrue(oWinnerHorizontal.isGameOver());
    GameBoard xWinnerDiagonal = GameSimulator.buildGameBoard(
          "332401133414455556110666004050633");
    assertTrue(xWinnerDiagonal.isGameOver());
  }
}