/**
 * A node contains data and references to the next and
 * the previous node.
 */
public class BiNode<E> {
  /** Data in the node. */
  private E data;
  /** Reference to the next node. */
  private BiNode<E> next;
  /** Reference to previous node. */
  private BiNode<E> previous;

  /**
   * Constructor initialises the current node with data and
   * sets references to the next and the previous node.
   * @param data_obj Data to be stored in the current node.
   * @param nextNodeRef  Reference to the next node to set
   *                     for the current node.
   * @param previousNodeRef Reference to the previous node to set
   *                        for the current node.
   * */
  public BiNode(E data_obj,
                BiNode<E> nextNodeRef, BiNode<E> previousNodeRef) {
    data = data_obj;
    next = nextNodeRef;
    previous = previousNodeRef;
  }

  /** Get data stored in the current node.
   *  @return Data in the current node */
  public E getData() { return data; }

  /** Set data in the current node. */
  public void setData(E obj) { data = obj; }

  /** Get reference to the next node in the current node.
   *  @return Reference to the next node */
  public BiNode<E> getNext() { return next; }

  /** Set reference to the next node in the current node. */
  public void setNext(BiNode<E> node) { next = node; }

  /** Get reference to the previous node in the current node.
   *  @return Reference to the previous node */
  public BiNode<E> getPrevious() { return previous; }

  /** Sets the reference to the previous node in the current node. */
  public void setPrevious(BiNode<E> node) { previous = node; }
}