import java.util.Arrays;

// Recursive sorting algorithm - Quicksort.
public class Quicksort {
  public static void main(String[] args) {
    String[] words = {
      "odd", "dub", "let", "etc", "his", "two", "lot", "who", "tip",
      "kit", "rat", "all", "ham", "lag", "bet", "got", "add", "pop",
      "god", "elk", "cat", "pig", "gas", "hat", "egg", "bun", "hog",
      "car", "sic", "bat", "one", "bag", "her", "ill", "fit", "pee",
      "for", "shy", "nob", "pie", "sun", "mac", "fun", "mad", "was",
      "sob", "mom", "gel", "new", "dam", "sat", "too", "sad", "old",
      "top", "bee", "ink", "put", "sea", "tie", "nor", "pea", "bit",
      "eve", "has", "not", "can", "dig", "hoe", "nod", "wet", "cab",
      "big", "nil", "saw", "pin", "she", "rip", "get", "yet", "see"
    };
    sort(words);
    System.out.println(Arrays.toString(words));
  }

  /** Sorts an array with Comparable elements. */
  public static <T extends Comparable<T>> void sort(T[] elementArray) {
    sort(elementArray, 0, elementArray.length - 1);
  }

  /** Sorts a segment (i.e. a subarray). */
  private static <T extends Comparable<T>> void
  sort(T[] segment, int first, int last) {

    /* Base case: 0 or one element in segment, no sorting necessary. */
    if (first >= last) {
      return;
    }

    // General case: partition the segment.
    int pivotPosition = performPartition(segment, first, last);

    // Recursive call to sort elements less than pivot element:
    sort(segment, first, pivotPosition-1);

    // Recursive call to sort elements greater than pivot element:
    sort(segment, pivotPosition+1, last);
  }

  /**
   * Partitions a segment.
   * Returns the index where the pivot element is to be found in the segment,
   * and hence the boundaries of the two smaller segments.
   */
  private static <T extends Comparable<T>> int
  performPartition(T[] segment, int first, int last) {
    // Choose the first element as pivot element:
    T pivotElement = segment[first];

    // The unpartitioned elements are always between
    // the lower and the upper indices, inclusive:
    int lower = first + 1;
    int upper = last;

    /* Repeat the loop until there are no more elements left to partition.
       The array has the following subarrays while the loop executes:
       first+1 .. lower-1: Elements with value less than the pivot element.
       lower .. upper: Elements not yet partitioned.
       upper+1 .. last: Elements with values greater than the pivot element.
     */
    while (lower <= upper) {
      T elementToPlace = segment[lower];
      boolean placedOK = elementToPlace.compareTo(pivotElement) <= 0;
      if (placedOK) {
        // Element is placed correctly at the lower index.
        // Decrease the unpartitioned segment size by increasing the lower index.
        ++lower;
      } else {
        // Swap elements at the ends of the unpartitioned segment.
        segment[lower] = segment[upper];
        segment[upper] = elementToPlace;
        // Element is now placed correctly at the upper index.
        // Decrease the unpartitioned segment size by decreasing the upper index.
        --upper;
      }
    }

    /* The segment is exhausted.
     * Place the pivot element correctly to mark the boundary between
     * elements smaller than the pivot element and
     * those that are greater than the pivot element.
     */
    int pivotPosition = lower - 1;
    segment[first] = segment[pivotPosition];
    segment[pivotPosition] = pivotElement;
    return pivotPosition;
  }
}
