// Recursive sorting algorithm - Quicksort.
public class QuicksortOriginal {
  public static void main(String[] args) {
    String[] words = {
      "odd", "dum", "let", "etc", "his", "two", "lot", "who", "tip",
      "kit", "rat", "all", "ham", "lag", "bet", "got", "add", "pop",
      "god", "elk", "cat", "pig", "gas", "hat", "egg", "bun", "hog",
      "car", "sic", "bat", "one", "bag", "her", "ill", "fit", "pee",
      "for", "shy", "nob", "pie", "sun", "mac", "fun", "mad", "was",
      "sob", "mom", "gel", "new", "dam", "sat", "too", "sad", "old",
      "top", "bee", "ink", "put", "sea", "tie", "nor", "pea", "bit",
      "eve", "has", "not", "can", "dig", "hoe", "nod", "wet", "cab",
      "big", "nil", "saw", "pin", "she", "rip", "get", "yet", "see"
    };
    sort(words);
    for (String word : words)
      System.out.println(word);
  }

  static <T extends Comparable<T>> void sort(T[] allElements) {
    quicksort(allElements, 0, allElements.length - 1);
  }

  static <T extends Comparable<T>> void quicksort(T[] elements,
                                                  int first, int last) {
    // Base case: 0 or one element, no sorting is necessary.
    if (first >= last)
      return;

    // General case:
    // We use the first element as pivot element
    T dividerElement = elements[first];
    /* The rest of the elements is in the unsplit segment of the array
       before we start the partitioning. */
    int firstUnsplit = first + 1;
    int lastUnsplit = last;

    /* Repeat this look until there are no more unsplit elements left.
       The array has the following segments while the loop executes:
          first+1 -- firstUnsplit-1: Elements less that the divider element.
          firstUnsplit -- lastUnsplit: Unsplit elements.
          lastUnsplit+1 -- last: Elements greater than the divider element.
    */
    while (firstUnsplit <= lastUnsplit) {
      T unsplitElement = elements[firstUnsplit];

      /* Ensure that the element is in the correct end of the unsplit
         segment and reduce the size of the segment to not include the
         element that has been correcltly placed. */
      boolean placeBeforeUnsplit = unsplitElement.compareTo(dividerElement) <= 0;
      if (placeBeforeUnsplit) {
        ++firstUnsplit;
      } else {
        // Swap the elements in the ends of the unsplit segment.
        swap(elements, firstUnsplit, lastUnsplit);
        // Move limit for elements greater than the divider element.
        --lastUnsplit;
      }
    }

    /* No unsplit elements left. Place the divider element between the split
       segments by swapping places with the last of the smaller elements. */
    int dividerIndex = firstUnsplit - 1;
    swap(elements, dividerIndex, first);

    // Rekursive kall
    quicksort(elements, first, dividerIndex-1);
    quicksort(elements, dividerIndex+1, last);
  }

  static <T> void swap(T[] elements, int i, int j) {
    T temp = elements[i];
    elements[i] = elements[j];
    elements[j] = temp;
  }
}
