import java.util.Scanner;

public class GreenHouseInstallation {

  public static void main(String[] args) {
    GreenHouse house = new GreenHouse();
    ClimateControl control = new ClimateControl(house);
    control.runClimateControl();
  }
}

class GreenHouse {

  final static int START_TEMPERATURE = 10;
  ClimateUnit climate;

  GreenHouse() {
    climate = new ClimateUnit(START_TEMPERATURE);
  }

  public void startAlarm() {
    System.out.println("ALARM! Evacuate the greenhouse!");
  }

  public int getDesiredTemperature() {
    Scanner keyboard = new Scanner(System.in);
    System.out.print("Type desired temperature: ");
    return keyboard.nextInt();
  }
}

class ClimateUnit {

  final static int UPPER_LIMIT_TEMP = 200;
  final static int LOWER_LIMIT_TEMP = -10;
  private int currentTemperature;

  public ClimateUnit(int startTemp) {
    currentTemperature = startTemp;
  }

  public int getTemperatur() {
    return currentTemperature;
  }

  public boolean isCritical() {
    return (currentTemperature > UPPER_LIMIT_TEMP)
    || (currentTemperature < LOWER_LIMIT_TEMP);
  }

  void incrementTemperature() throws Alarm {
    if (isCritical()) throw new Alarm("DANGER! DANGER!");
    ++currentTemperature;
  }

  void decrementTemperature() throws Alarm {
    if (isCritical()) throw new Alarm("DANGER! DANGER!");
    --currentTemperature;
  }
}

class Alarm extends Exception {
  public Alarm(String str) { super(str); }
}

class ClimateControl {
  final static int TEMP_VARIATION = 2;
  private GreenHouse house;
  int desiredTemperature;

  public ClimateControl(GreenHouse h) {
    house = h;
  }

  public void runClimateControl() {
    try {
      while (true) {
        if (isAdjustmentNecessary())
          adjustTemperature();
        else
          System.out.println("No adjusting av temperature: "
              + house.climate.getTemperatur());
      }
    } catch (Alarm u) {
      house.startAlarm();
    }
  }

  public boolean isAdjustmentNecessary() throws Alarm {
    if (house.climate.isCritical()) throw new Alarm("DANGER! DANGER!");
    desiredTemperature = house.getDesiredTemperature();
    return (Math.abs(desiredTemperature -
        house.climate.getTemperatur())) > TEMP_VARIATION;
  }

  public void adjustTemperature() throws Alarm {
    while ((Math.abs(desiredTemperature
        - house.climate.getTemperatur())) > TEMP_VARIATION) {
      if (desiredTemperature > house.climate.getTemperatur())
        house.climate.incrementTemperature();
      else
        house.climate.decrementTemperature();
    }
    System.out.println("Temperaturs adjusted to: "
        + house.climate.getTemperatur());
  }
}