import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

class CompanyUsingObjFiles {

  // Field
  private PersonnelRegister register;

  // Constructor
  CompanyUsingObjFiles() {
    register = new PersonnelRegister();
  }
  CompanyUsingObjFiles(PersonnelRegister register) {
    this.register = register;
  }

  // Write statistics
  void printReport() {
    System.out.println(register);
  }
  void printAllEmployeesToTerminalWindow() {
    Employee[] employees = register.getEmployeeArray();
    int numOfEmployees = register.getNumOfEmployees();
    System.out.println("Number of employees: " + numOfEmployees);
    for (int i = 0; i < numOfEmployees; i++) {
      System.out.println(employees[i]);
    }
  }

  /**
   * The method serialises employee objects.
   */
  void writeAllEmployeesToObjFile(String dataFileName, boolean append) {

    FileOutputStream outFileStream = null;
    ObjectOutputStream outObjStream = null;

    try {
      outFileStream = new FileOutputStream(dataFileName, append);  // (3)
      outObjStream = new ObjectOutputStream(outFileStream);        // (4)

      Employee[] employees = register.getEmployeeArray();
      int numOfEmployees = register.getNumOfEmployees();
      int numOfFemales = register.getNumOfFemales();
      outObjStream.writeObject(employees);                         // (5)
      outObjStream.writeInt(numOfEmployees);                       // (6)
      outObjStream.writeInt(numOfFemales);                         // (7)
    }
    catch (FileNotFoundException exception) {
      System.out.print("Cannot open file: " + dataFileName);
      System.exit(1);
    }
    catch (IOException exception) {
      System.out.print("Error writing to file: " + exception);
      System.exit(1);
    }
    finally {
      try {
        if (outObjStream != null)
          outObjStream.close();                                    // (8)
      } catch (IOException exception) {
        System.out.print("Error on closing: " + exception);
        System.exit(1);
      }
    }
  }

  /**
   * The method deserialises employee objects.
   */
  void readAllEmployeesFromObjFile(String dataFileName) {

    FileInputStream inFileStream = null;
    ObjectInputStream inObjStream = null;
    try {
      inFileStream = new FileInputStream(dataFileName);            // (9)
      inObjStream = new ObjectInputStream(inFileStream);           // (10)

      Employee[] employees = null;
      // Read an object from the file.
      Object newObject = inObjStream.readObject();                 // (11)
      // Check whether the object is of the right type.
      if (newObject instanceof Employee[])                         // (12)
        employees = (Employee[]) newObject; // Downcasting
      else
        System.out.println("Employee array not found.");
      int numOfEmployees = inObjStream.readInt();                  // (13)
      int numOfFemales = inObjStream.readInt();                    // (14)
      register = new PersonnelRegister(
                     employees, numOfEmployees, numOfFemales);     // (15)
    }
    catch (FileNotFoundException exception) {
      System.out.print("Cannot open file: " + dataFileName);
      System.exit(1);
    }
    catch (IOException exception) {
      System.out.print("Error reading from file: " + exception);
      System.exit(1);
    }
    catch (ClassNotFoundException exception) {
      System.out.print("Class definition not found: " + exception);
      System.exit(1);
    }
    finally {
      try {
        if (inObjStream != null)
          inObjStream.close();
      } catch (IOException exception) {
        System.out.print("Error on closing: " + exception);
        System.exit(1);
      }
    }
  }
}
