import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JTextField;

class IntegerDialogWindow extends JDialog implements ActionListener, WindowListener {

  // Reference to the owner window.
  private EchoWithDialog app;

  // Components.
  private JTextField inputField;
  private JButton okButton;

  public IntegerDialogWindow(EchoWithDialog frame) {

    // Call to the superclass constructor.                            (1)
    super(frame, "Integer Dialog Window", true); // (owner, title, modal)
    app = frame;

    // Create GUI components.
    okButton = new JButton("OK");
    inputField = new JTextField(20);
    inputField.setEditable(true);

    // Set up the GUI.
    Container contentPane = getContentPane();
    contentPane.add(inputField, BorderLayout.NORTH);
    contentPane.add(okButton, BorderLayout.SOUTH);

    // Register dialog window with the source (i.e. the okButton button)
    // in order to receive ActionEvents.
    okButton.addActionListener(this);

    // Register dialog with the source (i.e. with itself)
    // in order to receive WindowEvents.
    addWindowListener(this);

    // Show the GUI.
    pack();
    setVisible(true);
  }

  // Determine whether a string contains a valid integer.            (2)
  private boolean isValidInteger(String tf) {
    try {
      Integer.parseInt(tf);
    } catch (NumberFormatException ex) {
      return false;
    }
    return true;
  }

  // Terminate the dialog.                                           (3)
  private void removeDialogWindow() {
    setVisible(false);
    dispose();
  }

  // Methods from the ActionListener interface.                      (4)
  public void actionPerformed(ActionEvent event) {
    String integerString = inputField.getText();
    if (isValidInteger(integerString)) {
      app.showInteger(integerString);
      removeDialogWindow();
    }
  }

  // Methods from the WindowListener interface.
  public void windowClosing(WindowEvent event) {
    removeDialogWindow();
  }

  public void windowClosed(WindowEvent event)      { }
  public void windowOpened(WindowEvent event)      { }
  public void windowIconified(WindowEvent event)   { }
  public void windowDeiconified(WindowEvent event) { }
  public void windowActivated(WindowEvent event)   { }
  public void windowDeactivated(WindowEvent event) { }
}