package game;    // File: game/AbstractUserInterface.java

public abstract class AbstractUserInterface implements IUserInterface {
  protected GameBoard board;
  private boolean loopIsRunning;
  private boolean readyForNextMove;

  public void setBoard(GameBoard board) {                      // (1)
    this.board = board;
  }

  public void pieceWasDropped() {                              // (2)
    if (loopIsRunning) {
      readyForNextMove = true;
      showGameBoard();
    } else {
      runGameLoop();
    }
  }

  public void gameEndedInADraw() {                              // (3)
    showMessage("Game ended in a tie.");
  }

  public void playerHasWon(IPieceIdentity piece) {             // (4)
    showMessage(identityAsText(piece) + " has won");
  }

  public void runGameLoop() {                                  // (5)
    readyForNextMove = true;
    showGameBoard();
    loopIsRunning = true;
    while (readyForNextMove && !board.isGameOver())
      playOneMove();
    loopIsRunning = false;
  }

  private void playOneMove() {                                 // (6)
    IPieceIdentity piece = board.nextPiece();
    showNextPiece(piece);
    ParticipantIdentity participant = (ParticipantIdentity) piece;
    readyForNextMove = false;
    participant.performMove(board);
  }

  protected void showNextPiece(IPieceIdentity piece) {         // (7)
    showMessage("Next move: " + identityAsText(piece));
  }

  protected String identityAsText(IPieceIdentity piece) {      // (8)
    ParticipantIdentity participant = (ParticipantIdentity) piece;
    return participant.textRepresentation;
  }

  protected abstract void showGameBoard();
  protected abstract void showMessage(String text);
}