package game.swing;    // File: game/swing/GameSwingUI.java
import game.AbstractUserInterface;
import game.GameBoard;
import game.IPieceIdentity;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

public class GameSwingUI extends AbstractUserInterface {
  Container boardComponents;
  Container topBar;
  JButton newGame;
  JLabel message;
  JFrame frame;

  GameSwingUI() {
    frame = new JFrame("Four-in-a-row");

    Container window = frame.getContentPane();
    window.setLayout(new BorderLayout());

    topBar = new JPanel();
    topBar.setLayout(new FlowLayout(FlowLayout.LEFT));
    message = new JLabel("New game started");
    newGame = new JButton("New Game");
    ActionListener newGameAction = new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        board.reset();
        runGameLoop();
      }
    };
    newGame.addActionListener(newGameAction);

    topBar.add(newGame);
    topBar.add(message);
    window.add(topBar, BorderLayout.NORTH);

    boardComponents = new JPanel();
    window.add(boardComponents, BorderLayout.CENTER);
  }

  public void setBoard(GameBoard board) {
    super.setBoard(board);

    int columns = GameBoard.COLUMN_COUNT;
    int rows = GameBoard.ROW_COUNT;
    boardComponents.setLayout(new GridLayout(columns, rows + 1));

    for (int column = 0; column < columns; ++column) {
      boardComponents.add(makeDropButton(column));
    }

    for (int row = 0; row < GameBoard.ROW_COUNT; ++row) {
      for (int column = 0; column < columns; ++column) {
        JComponent cell = new JLabel("");
        cell.setBorder(BorderFactory.createEtchedBorder());
        cell.setOpaque(true);
        boardComponents.add(cell);
      }
    }
  }

  Component makeDropButton(final int column) {
    JButton dropButton = new JButton("Drop");
    ActionListener dropAction = new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        board.dropPieceDownColumn(board.nextPiece(), column);
      }
    };
    dropButton.addActionListener(dropAction);
    return dropButton;
  }

  protected void showGameBoard() {
    for (int column = 0; column < GameBoard.COLUMN_COUNT; ++column) {
      setDropPermissionForColumn(column, false);

      for (int row = 0; row < GameBoard.ROW_COUNT; ++row) {
        IPieceIdentity piece = board.pieceAt(column, row);
        ComponentColorIdentity identity = (ComponentColorIdentity) piece;
        Component component = componentForCell(column, row);
        identity.applyComponentColor(component);
      }
    }
  }

  void setDropPermissionForColumn(int column, boolean dropIsAllowed) {
    Component button = boardComponents.getComponent(column);
    button.setEnabled(dropIsAllowed);
  }

  Component componentForCell(int column, int row) {
    return boardComponents.getComponent((GameBoard.ROW_COUNT - row) *
           GameBoard.COLUMN_COUNT + column);
  }
  protected void showMessage(String text) {
    message.setText(text);
  }

  public void startUserInterface() {
    runGameLoop();
    frame.pack();
    frame.setVisible(true);
  }
}