package museumPkg;

import java.awt.Container;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;

public class MainMuseumWindow extends JFrame implements ActionListener {

  // Buttons
  protected JButton newButton;
  protected JButton updateButton;
  protected JButton removeButton;
  protected JButton exitButton;

  // Text fields
  protected JLabel leftLabel;
  protected JLabel rightLabel;

  // Museum that will be maintained.
  protected Museum museum = new Museum();

  /**
   * Constructor for main museum window
   */
  public MainMuseumWindow() {
    super("GUI Museum Program");

    //Create buttons
    newButton = new JButton("New Museum Item");
    updateButton = new JButton("Update Museum Item");
    removeButton = new JButton("Delete Museum Item");
    exitButton = new JButton("Exit");

    //Create text fields
    leftLabel = new JLabel("Number: ", Label.RIGHT);
    rightLabel = new JLabel("0");

    //Create a museum
    museum = new Museum();

    //Register the listener (main window) with the buttons.
    newButton.addActionListener(this);
    updateButton.addActionListener(this);
    removeButton.addActionListener(this);
    exitButton.addActionListener(this);

    //Create component hierarchy in the frame.
    Container content = getContentPane();
    content.setLayout(new GridLayout(3, 2));
    content.add(newButton);
    content.add(updateButton);
    content.add(removeButton);
    content.add(exitButton);
    content.add(leftLabel);
    content.add(rightLabel);

    pack();
    setSize(300, 200);
    setVisible(true);
  }

  //Implements the interface ActionListener.
  /**
   * Called if a button is clicked.
   */
  public void actionPerformed(ActionEvent e) {
    if (e.getSource() == newButton) {
      new NewItemFrame(this);
    } else if (e.getSource() == updateButton) {
      new UpdateItemFrame(this);
    } else if (e.getSource() == removeButton) {
      new RemoveItemFrame(this);
    } else if (e.getSource() == exitButton) {
      museum.writeCollectionToFile();
      setVisible(false);
      dispose();
      System.exit(0);
    }
  }

  /**
   * @returns Returns reference to the museum.
   */
  public Museum getMuseum() {
    return museum;
  }

  /**
   * Shows updated number of items.
   */
  public void setNumOfItems() {
    rightLabel.setText("" + museum.numberOfItems());
  }

  /**
   * Create a new MainMuseumWindow object.
   */
  public static void main(String[] args) {
    MainMuseumWindow mmw = new MainMuseumWindow();
    mmw.setNumOfItems();
  }
}
