package museumPkg;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

/**
 *  This class maintians a collection of items.
 **/
public class Museum {

  // Instance members
  /** Name of file with information about museum items. */
  private static String dataFileName = "museum.dat";

  /** List to maintain items */
  LinkedList<MuseumItem> itemList = null;

  /**
   *  Reads items from a file.
   *  If not successful, creates an empty museum.
   **/
  public Museum() {
    if (!readCollectionFromFile())
      itemList = new LinkedList<MuseumItem>();
  }

  /**
   *  Reads items from a file.
   *  @return false, if any problems with reading.
   **/
  boolean readCollectionFromFile() {
    try {
      FileInputStream fileIS = new FileInputStream(dataFileName);
      ObjectInputStream objIS = new ObjectInputStream(fileIS);
      itemList = (LinkedList<MuseumItem>) objIS.readObject();
      objIS.close();
    } catch (Exception exception) {
      return false;
    }
    return true;
  }

  /**
   *  Writes collection to a file.
   **/
  public void writeCollectionToFile() {
    try {
      FileOutputStream fileOS = new FileOutputStream(dataFileName);
      ObjectOutputStream objOS = new ObjectOutputStream(fileOS);
      objOS.writeObject(itemList);        // Writes the whole list.
      objOS.close();
    } catch (Exception exception) {
      System.err.println("Error when writing: " + exception);
    }
  }

  /**
   * Create an array of items from the list of items.
   * @return Arry with all items.
   **/
  public MuseumItem[] hentSamling() {
    MuseumItem[] itemArray = new MuseumItem[itemList.numberOfNodes()];
    itemList.listToArray(itemArray);
    return itemArray;
  }

  /**
   *  @return Number of items in the museuem.
   **/
  public int numberOfItems() {
    return itemList.numberOfNodes();
  }

  /**
   *  Finds an item in the museum.
   *  @param refNum Reference number of item to find.
   *  @return The item with the specified reference number, or null if not
   *  found.
   **/
  public MuseumItem findItem(int refNum) {
    Node<MuseumItem> node = itemList.find(new MuseumItem(refNum, "X", "X"));
    if (node != null)
      return node.getData();
    return null;
  }

  /**
   *  Register an item in the museum.
   *  @param museumItem  The item to be registered.
   **/
  public void registerItem(MuseumItem museumItem) {
    itemList.insertAtHead(museumItem);
  }

  /**
   *  Remove an item from the museum.
   *  @param museumItem  The item to be removed.
   **/
  public void removeItem(MuseumItem museumItem) {
    itemList.remove(museumItem);
  }
}
