package museumPkg;

/**
 * GUI to change information about an exiting item in the museum.
 */
public class UpdateItemFrame extends AbstractItemFrame {

  private MuseumItem currentItem = null;

  /**
   * "Ovverrides" the runButton of the superclass AbstractItemFrame.
   * Name and description cannot be edited the first time in order to
   * lookup the item based on the reference number.
   */
  public UpdateItemFrame(MainMuseumWindow mainWindow) {
    super(mainWindow);
    setTitle("Update Museum Item");

    runButton.setText("Update");
    nameField.setEditable(false);
    descriptionField.setEditable(false);
  }

  /**
   * Ovverides the method from the superclass AbstractItemFrame.
   * @exception MuseumItemException  If error during update.
   */
  void doAction() throws MuseumItemException {
    if (currentItem == null)
      findItemInMuseum();
    else {
      currentItem.setName(nameField.getText());
      currentItem.setDescription(descriptionField.getText());
      removeFrame();
    }
  }

  /**
   * Ovverides the method from the superclass AbstractItemFrame.
   * Finds the item with the reference number provided.
   * @exception MuseumItemException If error during update.
   */
  void findItemInMuseum() throws MuseumItemException {
    currentItem = mainWindow.getMuseum().findItem(getItemReferenceNum());
    // Set the information in the text fields is updated, and field
    // made editable.
    try {
      nameField.setText(currentItem.getName());
      descriptionField.setText(currentItem.getDescription());

      nameField.setEditable(true);
      descriptionField.setEditable(true);
    } catch (NullPointerException e) {
      throw new MuseumItemException("Item not found.");
    }
  }
}
